@@ -5,17 +5,20 @@ library. Reference: https://cryptography.io/en/latest/fernet/
 """
 
 import base64 as _base64
+import logging
 import os as _os
-import warnings as _warnings
+from warnings import warn as _warn
 
 import cryptography.fernet as _fernet
-import cryptography.hazmat.backends as _backends
 import cryptography.hazmat.primitives.ciphers as _ciphers
 
 from .errors import SymmetricCryptographyError
 
 
-class Fernet(object):
+logger = logging.getLogger(__name__)
+
+
+class Fernet:
     """The default symmetric cryptography method."""
 
     @staticmethod
@@ -33,9 +36,7 @@ class Fernet(object):
         :param key: byte data representing the encyption/decryption key
         """
         if key:
-            fernet_key_error = SymmetricCryptographyError(
-                "Fernet key must be 32 url-safe base64-encoded bytes."
-            )
+            fernet_key_error = SymmetricCryptographyError("Fernet key must be 32 url-safe base64-encoded bytes.")
             try:
                 raw_key = _base64.b64decode(key)
             except Exception as e:
@@ -61,7 +62,8 @@ class Fernet(object):
                 "Remove any other arguements. "
                 "In the next version, this method will not allow them."
             )
-            _warnings.warn(_deprecation_msg, DeprecationWarning)
+            logger.warning(_deprecation_msg)
+            _warn(_deprecation_msg, DeprecationWarning)
 
         ciphertext = self._symmetric.encrypt(plaintext)
         return ciphertext
@@ -79,7 +81,8 @@ class Fernet(object):
                 "Remove any other arguements. "
                 "In the next version, this method will not allow them."
             )
-            _warnings.warn(_deprecation_msg, DeprecationWarning)
+            logger.warning(_deprecation_msg)
+            _warn(_deprecation_msg, DeprecationWarning)
 
         plaintext = self._symmetric.decrypt(ciphertext)
         return plaintext
@@ -90,18 +93,19 @@ class Fernet(object):
             "Remove any calls to this method. "
             "In the next version, this method will be removed."
         )
-        _warnings.warn(_deprecation_msg, DeprecationWarning)
+        logger.warning(_deprecation_msg)
+        _warn(_deprecation_msg, DeprecationWarning)
 
 
-class AESCipher(object):
+class AESCipher:
     """[deprecated] Symmetric cryptography method using AES.
 
     The default parameter set is AES 128bit in CBC mode.
     """
 
     POSTFIX_MODE = {
-        'cbc': _ciphers.modes.CBC,
-        'cfb': _ciphers.modes.CFB,
+        "cbc": _ciphers.modes.CBC,
+        "cfb": _ciphers.modes.CFB,
     }
 
     AES_BLOCK_SIZE = int(_ciphers.algorithms.AES.block_size / 8)
@@ -110,13 +114,14 @@ class AESCipher(object):
     def _deprecation_notice(cls):
         """Warn about deprecation of this class."""
         _deprecation_msg = (
-            '{name} {type} is deprecated. '
-            'It will be removed in the next version. '
-            'Use saml2.cryptography.symmetric.Default '
-            'or saml2.cryptography.symmetric.Fernet '
-            'instead.'
+            "{name} {type} is deprecated. "
+            "It will be removed in the next version. "
+            "Use saml2.cryptography.symmetric.Default "
+            "or saml2.cryptography.symmetric.Fernet "
+            "instead."
         ).format(name=cls.__name__, type=type(cls).__name__)
-        _warnings.warn(_deprecation_msg, DeprecationWarning)
+        logger.warning(_deprecation_msg)
+        _warn(_deprecation_msg, DeprecationWarning)
 
     def __init__(self, key):
         """
@@ -126,39 +131,35 @@ class AESCipher(object):
         self.__class__._deprecation_notice()
         self.key = key
 
-    def build_cipher(self, alg='aes_128_cbc'):
+    def build_cipher(self, alg="aes_128_cbc"):
         """
         :param alg: cipher algorithm
         :return: A Cipher instance
         """
         self.__class__._deprecation_notice()
-        typ, bits, cmode = alg.lower().split('_')
+        typ, bits, cmode = alg.lower().split("_")
         bits = int(bits)
         iv = _os.urandom(self.AES_BLOCK_SIZE)
 
         if len(iv) != self.AES_BLOCK_SIZE:
-            raise Exception('Wrong iv size: {}'.format(len(iv)))
+            raise Exception(f"Wrong iv size: {len(iv)}")
 
         if bits not in _ciphers.algorithms.AES.key_sizes:
-            raise Exception('Unsupported key length: {}'.format(bits))
+            raise Exception(f"Unsupported key length: {bits}")
 
         if len(self.key) != bits / 8:
-            raise Exception('Wrong Key length: {}'.format(len(self.key)))
+            raise Exception(f"Wrong Key length: {len(self.key)}")
 
         try:
             mode = self.POSTFIX_MODE[cmode]
         except KeyError:
-            raise Exception('Unsupported chaining mode: {}'.format(cmode))
+            raise Exception(f"Unsupported chaining mode: {cmode}")
 
-        cipher = _ciphers.Cipher(
-                _ciphers.algorithms.AES(self.key),
-                mode(iv),
-                backend=_backends.default_backend())
+        cipher = _ciphers.Cipher(_ciphers.algorithms.AES(self.key), mode(iv))
 
         return cipher, iv
 
-    def encrypt(self, msg, alg='aes_128_cbc', padding='PKCS#7', b64enc=True,
-                block_size=AES_BLOCK_SIZE):
+    def encrypt(self, msg, alg="aes_128_cbc", padding="PKCS#7", b64enc=True, block_size=AES_BLOCK_SIZE):
         """
         :param key: The encryption key
         :param msg: Message to be encrypted
@@ -168,9 +169,9 @@ class AESCipher(object):
         :return: The encrypted message
         """
         self.__class__._deprecation_notice()
-        if padding == 'PKCS#7':
+        if padding == "PKCS#7":
             _block_size = block_size
-        elif padding == 'PKCS#5':
+        elif padding == "PKCS#5":
             _block_size = 8
         else:
             _block_size = 0
@@ -191,7 +192,7 @@ class AESCipher(object):
 
         return enc_msg
 
-    def decrypt(self, msg, alg='aes_128_cbc', padding='PKCS#7', b64dec=True):
+    def decrypt(self, msg, alg="aes_128_cbc", padding="PKCS#7", b64dec=True):
         """
         :param key: The encryption key
         :param msg: Base64 encoded message to be decrypted
@@ -202,9 +203,9 @@ class AESCipher(object):
 
         cipher, iv = self.build_cipher(alg=alg)
         decryptor = cipher.decryptor()
-        res = decryptor.update(data)[self.AES_BLOCK_SIZE:]
+        res = decryptor.update(data)[self.AES_BLOCK_SIZE :]
         res += decryptor.finalize()
-        if padding in ['PKCS#5', 'PKCS#7']:
+        if padding in ["PKCS#5", "PKCS#7"]:
             idx = bytearray(res)[-1]
             res = res[:-idx]
         return res
