@@ -1,9 +1,10 @@
-from six.moves.urllib import parse
+from urllib import parse
 
 from saml2.entity import Entity
 from saml2.response import VerificationError
 
-__author__ = 'rolandh'
+
+__author__ = "rolandh"
 
 IDPDISC_POLICY = "urn:oasis:names:tc:SAML:profiles:SSO:idp-discovery-protocol:single"
 
@@ -24,10 +25,10 @@ class DiscoveryServer(Entity):
 
         # verify
 
-        for key in ["isPassive", "return", "returnIDParam", "policy",
-                    'entityID']:
+        for key in ["isPassive", "return", "returnIDParam", "policy", "entityID"]:
             try:
-                assert len(dsr[key]) == 1
+                if len(dsr[key]) != 1:
+                    raise Exception(f"Invalid DS request keys: {key}")
                 dsr[key] = dsr[key][0]
             except KeyError:
                 pass
@@ -37,9 +38,11 @@ class DiscoveryServer(Entity):
             if part.query:
                 qp = parse.parse_qs(part.query)
                 if "returnIDParam" in dsr:
-                    assert dsr["returnIDParam"] not in qp.keys()
+                    if dsr["returnIDParam"] in qp.keys():
+                        raise Exception("returnIDParam value should not be in the query params")
                 else:
-                    assert "entityID" not in qp.keys()
+                    if "entityID" in qp.keys():
+                        raise Exception("entityID should not be in the query params")
         else:
             # If metadata not used this is mandatory
             raise VerificationError("Missing mandatory parameter 'return'")
@@ -47,17 +50,16 @@ class DiscoveryServer(Entity):
         if "policy" not in dsr:
             dsr["policy"] = IDPDISC_POLICY
 
-        try:
-            assert dsr["isPassive"] in ["true", "false"]
-        except KeyError:
-            pass
+        is_passive = dsr.get("isPassive")
+        if is_passive not in ["true", "false"]:
+            raise ValueError(f"Invalid value '{is_passive}' for attribute 'isPassive'")
 
         if "isPassive" in dsr and dsr["isPassive"] == "true":
             dsr["isPassive"] = True
         else:
             dsr["isPassive"] = False
 
-        if not "returnIDParam" in dsr:
+        if "returnIDParam" not in dsr:
             dsr["returnIDParam"] = "entityID"
 
         return dsr
@@ -65,9 +67,7 @@ class DiscoveryServer(Entity):
     # -------------------------------------------------------------------------
 
     @staticmethod
-    def create_discovery_service_response(return_url=None,
-                                          returnIDParam="entityID",
-                                          entity_id=None, **kwargs):
+    def create_discovery_service_response(return_url=None, returnIDParam="entityID", entity_id=None, **kwargs):
         if return_url is None:
             return_url = kwargs["return"]
 
@@ -77,9 +77,9 @@ class DiscoveryServer(Entity):
             part = parse.urlparse(return_url)
             if part.query:
                 # Iff there is a query part add the new info at the end
-                return_url = "%s&%s" % (return_url, qp)
+                return_url = f"{return_url}&{qp}"
             else:
-                return_url = "%s?%s" % (return_url, qp)
+                return_url = f"{return_url}?{qp}"
 
         return return_url
 
@@ -93,10 +93,6 @@ class DiscoveryServer(Entity):
 
     def verify_return(self, entity_id, return_url):
         for endp in self.metadata.discovery_response(entity_id):
-            try:
-                assert return_url.startswith(endp["location"])
-            except AssertionError:
-                pass
-            else:
+            if not return_url.startswith(endp["location"]):
                 return True
         return False
